<?php

namespace Reinos\Mollie\Core\Action;

/**
 * @author        Rein de Vries <support@reinos.nl>
 * @link        http://addons.reinos.nl
 * @copyright    Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * @license     http://addons.reinos.nl/commercial-license
 *
 * Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Rein de Vries and
 * Reinos.nl Internet Media) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */
class AddToCart extends AbstractAction
{
    public function execute()
    {
        //set the defaults
        $qty = (int)ee()->input->post('qty') ? ee()->input->post('qty') : 0;
        $name = ee()->input->post('product_name');
        $price = ee()->input->post('product_price');
        $tax = ee()->input->post('tax');
        $sku = ee()->input->post('sku');
        $entry_id = ee()->input->post('entry_id');
        $disable_shipping = ee()->input->post('disable_shipping');
        $clear_cart = ee()->input->post('clear_cart');

        // custom fields
        $custom_fields = ee()->input->post('custom_fields');
        $custom_fields_from_tag = unserialize(base64_decode(ee()->input->post('custom_fields_from_tag')));

        //merge them together
        $custom_fields = base64_encode(serialize(array_merge($custom_fields, $custom_fields_from_tag)));

        //check the price, this cannot be empty and should be an float
        if (filter_var($price, FILTER_VALIDATE_FLOAT) === false) {
            ee()->session->set_flashdata(REINOS_MOLLIE_MAP . '_invalid_price_' . $sku, 'invalid_price');
            ee()->functions->redirect(ee()->input->post('return_url'));
        }

        //remove other cart items
        if ($clear_cart) {
            ee(REINOS_MOLLIE_SERVICE_NAME . ':Cart')->clear();
        }

        //add a new one
        ee(REINOS_MOLLIE_SERVICE_NAME . ':Cart')->add($name, $price, $tax, $qty, $sku, $disable_shipping, $entry_id,
            $custom_fields);

        //redirect
        ee()->functions->redirect(ee()->input->post('return_url'));
    }
}
