<?php

namespace Reinos\Mollie\Core\Action;

/**
 * @author        Rein de Vries <support@reinos.nl>
 * @link        http://addons.reinos.nl
 * @copyright    Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * @license     http://addons.reinos.nl/commercial-license
 *
 * Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Rein de Vries and
 * Reinos.nl Internet Media) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */
class Redirect extends AbstractAction
{
    public function execute()
    {

        if (isset($_GET['hash'])) {
            //get the order
            $order = ee(REINOS_MOLLIE_SERVICE_NAME . ':Payment')->getPaymentForHash(ee()->input->get("hash"));
            if ($order === false) {
                header("Location: " . ee()->config->item('site_url'));
                exit;
            }

            //Check if we handle it with Mollie or just manuel
            if (ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('mollie_mode') != 'none') {
                //Init Mollie
                ee(REINOS_MOLLIE_SERVICE_NAME . ':Payment')->init();

                //set the status
                $status = ee(REINOS_MOLLIE_SERVICE_NAME . ':Payment')->checkStatus($order->transaction_id);

                //@todo wanneer de status open is, een aparte mail sturen met dat er nog betaald moet worden.
                if ($status == 'paid' || $status == 'open') {
                    //finish the call and clear cart, send mail and redirect.
                    $this->finish($order);
                } else {
                    if ($status == 'canceled' || $status == 'failed') {
                        header("Location: " . $order->checkout_url . '?status=' . $status);
                    } else {
                        header("Location: " . ee()->config->item('site_url'));
                    }
                }
            } else {
                //finish the call and clear cart, send mail and redirect.
                $this->finish($order);
            }
        } else {
            header("Location: " . ee()->config->item('site_url'));
        }

        exit;
    }

    //------------------------------------------------------------------------------------------------

    private function finish($order)
    {
        //clear the cart
        ee(REINOS_MOLLIE_SERVICE_NAME . ':Cart')->clearAll();

        //send the mail
        ee(REINOS_MOLLIE_SERVICE_NAME . ':Email')->sendEmail($order);

        //redirect to the order detail
        header("Location: " . reduce_double_slashes($order->return_url . '/' . $order->hash));
    }
}
