<?php

namespace Reinos\Mollie\Core\Tag;

/**
 * @author        Rein de Vries <support@reinos.nl>
 * @link        http://addons.reinos.nl
 * @copyright    Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * @license     http://addons.reinos.nl/commercial-license
 *
 * Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Rein de Vries and
 * Reinos.nl Internet Media) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */
class AddToCart extends AbstractTag
{
    public function parse()
    {
        //license check
        if (ee(REINOS_MOLLIE_SERVICE_NAME . ':License')->hasValidLicense()) {
            $sku = ee()->TMPL->fetch_param('sku', uniqid());
            $entry_id = ee()->TMPL->fetch_param('entry_id', 0);
            $product_name = ee()->TMPL->fetch_param('product_name', '');
            $product_price = ee()->TMPL->fetch_param('product_price', '');
            $tax = ee()->TMPL->fetch_param('tax');
            $custom_fields = ee(REINOS_MOLLIE_SERVICE_NAME . ':Helper')->filter_by_key_prefix(ee()->TMPL->tagparams,
                'custom_fields:', true);

            //set the vars
            $return = '';
            $variables = array(
                'error:invalid_price' => 0
            );

            //get attributes
            $_attr = preg_grep('/^form_attr:(.*)$/i', array_keys(ee()->TMPL->tagparams));
            $attr = array();
            if (!empty($_attr)) {
                foreach ($_attr as $a) {
                    $name = str_replace('form_attr:', '', $a);
                    $attr[$name] = ee()->TMPL->tagparams[$a];
                }
            }

            //has an error for invalid_price?
            $invalid_price = ee()->session->flashdata(REINOS_MOLLIE_MAP . '_invalid_price_' . $sku);
            if ($invalid_price == 'invalid_price') {
                $variables['error:invalid_price'] = 1;
            }

            //set the tagdata
            $tagdata = ee()->TMPL->tagdata;

            // start our form output
            $return .= ee(REINOS_MOLLIE_SERVICE_NAME . ':Helper')->form_open(array(
                'hidden_fields' => array(
                    'return_url' => ee()->functions->create_url(ee()->TMPL->fetch_param('return_url',
                        ee()->uri->uri_string)),
                    'current_url' => ee()->functions->create_url(ee()->TMPL->fetch_param('current_url',
                        ee()->uri->uri_string)),
                    'sku' => $sku,
                    'entry_id' => $entry_id,
                    'product_name' => $product_name,
                    'product_price' => $product_price,
                    'tax' => $tax,
                    'clear_cart' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Helper')->check_yes(ee()->TMPL->fetch_param('clear_cart',
                        'no')),
                    'disable_shipping' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Helper')->check_yes(ee()->TMPL->fetch_param('disable_shipping',
                        'no')),
                    'custom_fields_from_tag' => base64_encode(serialize($custom_fields))
                ),
                'action' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->act_url . '&method=add_to_cart',
                'id' => ee()->TMPL->fetch_param('form_id', 'reinos_mollie-add_to_cart'),
                'class' => ee()->TMPL->fetch_param('form_class', ''),
                'attr' => ee()->TMPL->fetch_param('attr', $attr)
            ));

            //parse the variables
            $return .= ee()->TMPL->parse_variables($tagdata, array($variables));

            $return .= "</form>";

            return $return;
        } else {
            ee(REINOS_MOLLIE_SERVICE_NAME . ':Log')->add_log('Your license key appear to be invalid. Please fill in the license in the module CP.',
                'error');
        }
    }
}
