<?php

namespace Reinos\Mollie\Core\Tag;

/**
 * @author        Rein de Vries <support@reinos.nl>
 * @link        http://addons.reinos.nl
 * @copyright    Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * @license     http://addons.reinos.nl/commercial-license
 *
 * Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Rein de Vries and
 * Reinos.nl Internet Media) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */
class Checkout extends AbstractTag
{
    public function parse()
    {
        //license check
        if (ee(REINOS_MOLLIE_SERVICE_NAME . ':License')->hasValidLicense()) {
            //set the vars
            $return = '';
            $variables = array(
                'has_field_errors' => 0,
                'has_global_errors' => 0,
                'global_error' => '',
            );

            //set the values
            $variables = array_merge($variables, ee(REINOS_MOLLIE_SERVICE_NAME . ':Checkout')->getFieldValues());

            //check if we have errors
            $errors = ee()->session->flashdata(REINOS_MOLLIE_SERVICE_NAME . ':errors');

            //grab the no items error if present
            if (isset($errors['no_items_to_checkout'])) {
                //has error marker
                $variables['has_global_errors'] = 1;

                //set the error
                $variables['global_error'] = $errors['no_items_to_checkout'];

                //delete from the error array
                unset($errors['no_items_to_checkout']);
            } //handle other errors
            else {
                if (!empty($errors)) {
                    //has error marker
                    $variables['has_field_errors'] = 1;

                    foreach ($errors as $key => $error) {
                        $variables[$key . ':error'] = $error;
                    }
                }
            }

            // start our form output
            $return .= ee(REINOS_MOLLIE_SERVICE_NAME . ':Helper')->form_open(array(
                'hidden_fields' => array(
                    'return_url' => ee()->functions->create_url(ee()->TMPL->fetch_param('return_url',
                        ee()->uri->uri_string)),
                    'current_url' => ee()->functions->create_url(ee()->TMPL->fetch_param('current_url',
                        ee()->uri->uri_string)),
                ),
                'action' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->act_url . '&method=checkout',
                'id' => ee()->TMPL->fetch_param('form_id', 'reinos-mollie-checkout'),
                'class' => ee()->TMPL->fetch_param('form_class', '')
            ));

            //parse the variables
            $return .= ee()->TMPL->parse_variables(ee()->TMPL->tagdata, array($variables));

            $return .= "</form>";

            return $return;
        } else {
            ee(REINOS_MOLLIE_SERVICE_NAME . ':Log')->add_log('Your license key appear to be invalid. Please fill in the license in the module CP.',
                'error');
        }
    }
}
