<?php

namespace Reinos\Mollie\Core\Tag;

/**
 * @author        Rein de Vries <support@reinos.nl>
 * @link        http://addons.reinos.nl
 * @copyright    Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * @license     http://addons.reinos.nl/commercial-license
 *
 * Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Rein de Vries and
 * Reinos.nl Internet Media) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */
class Order extends AbstractTag
{

    public function parse()
    {
        //fetch the prefix if needed
        $prefix = ee()->TMPL->fetch_param('prefix', 'order:');
        $order_id = ee()->TMPL->fetch_param('order_id', 0);
        $hash = ee()->TMPL->fetch_param('hash', 0);

        //set the empty variable array
        $variables = [];

        //fetch the order
        $order = ee('Model')->get(REINOS_MOLLIE_MAP . ':Order');

        if ($order_id != '') {
            $order->filter('order_id', $order_id);
        } else {
            if ($hash != '') {
                $order->filter('hash', $hash);
            } else {
                return ee()->TMPL->no_results();
            }
        }

        if ($order->count() > 0) {
            //get the first one
            $order = $order->first();

            foreach ($order->toArray() as $field => $value) {
                //format price if needed
                $value = ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->needFloatConverting($field, $value, true);

                //also check the float on the items and add a prefix
                if ($field === 'items') {
                    foreach ($value as $item_k => $item_v) {
                        //calculate totals
                        $item_v['price_total'] = $item_v['qty'] * $item_v['price'];
                        $item_v['price_total_plus_tax'] = $item_v['qty'] * $item_v['price_plus_tax'];

                        foreach ($item_v as $k => $v) {
                            $value[$item_k][$prefix . 'items:' . $k] = ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->needFloatConvertingProduct($k,
                                $v, true);
                        }
                    }
                }

                $variables[$prefix . $field] = $value;
            }

            //return the parsed data
            return ee()->TMPL->parse_variables(ee()->TMPL->tagdata, array($variables));
        }

        return ee()->TMPL->no_results();
    }
}
