<?php

/**
 * @author        Rein de Vries <support@reinos.nl>
 * @link        http://addons.reinos.nl
 * @copyright    Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * @license     http://addons.reinos.nl/commercial-license
 *
 * Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Rein de Vries and
 * Reinos.nl Internet Media) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */

namespace Reinos\Mollie\Service;

require_once PATH_THIRD . 'reinos_mollie/config.php';

class Cart
{

    public $tax = 0;
    public $taxType = 'none';
    public $shipping = 0;

    public function __construct()
    {
        if (session_id() == "") {
            session_start();
        }

        $this->taxType = ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('tax_type');
    }

    //---------------------------------------------------------------------------------------

    public function add(
        $name,
        $price,
        $tax,
        $qty,
        $sku = '',
        $disable_shipping = 0,
        $entry_id = 0,
        $custom_fields = false
    ) {
        //set the key
        $sku = $sku == '' ? uniqid() : $sku;

        // if we deal with a named tax, fetch the value from the setting
        // otherwise its a user given tax value
        if (!is_numeric($tax)) {
            $tax = $this->getTax($tax);
        }

        $price = $this->calcTax($price, (int)$tax);

        //set the data
        $new_data = $compare_data = array(
            'id' => $sku . '_' . uniqid(),
            'name' => $name,
            'price' => $price['price_excl'],
            'price_plus_tax' => $price['price_incl'],
            'sku' => $sku,
            'entry_id' => $entry_id,
            'disable_shipping' => $disable_shipping,
            'custom_fields' => $custom_fields,
            'qty' => $qty,
        );

        // for the compare we are not using the QTY
        unset($compare_data['qty']);
        unset($compare_data['id']);

        // marker for adding a new product
        $add_new_product = false;

        //already in cart?
        if (!isset($_SESSION[REINOS_MOLLIE_MAP]['products'][$sku])) {
            $_SESSION[REINOS_MOLLIE_MAP]['products'][$sku] = [];
            $add_new_product = true;
        }

        // check in the array if we need to update or set a marker for adding
        foreach ($_SESSION[REINOS_MOLLIE_MAP]['products'][$sku] as $key => &$current_data) {
            //default is add, only if found then update
            $add_new_product = true;

            $current_compare_data = $current_data;
            unset($current_compare_data['qty']);
            unset($current_compare_data['id']);

            // Update QTY if they are equal
            if ($current_compare_data == $compare_data) {
                $current_data['qty'] = $qty + 1;
                $add_new_product = false;
                break;
            }
        }

        //add a new row if needed
        if ($add_new_product) {
            $_SESSION[REINOS_MOLLIE_MAP]['products'][$sku][] = $new_data;
        }

    }

    //---------------------------------------------------------------------------------------

    public function update($id, $values = array())
    {
        $product = $this->_find($id);

        if (!empty($product)) {
            //if the qty is set to 0, then delete it
            if (isset($values['qty']) && $values['qty'] == 0) {
                $this->_delete($product['sku'], $product['id']);
            } else {
                $this->_update($product['sku'], $product['id'], $values);
            }
        }
    }

    //---------------------------------------------------------------------------------------

    private function _delete($sku, $id)
    {
        if (isset($_SESSION[REINOS_MOLLIE_MAP]['products'][$sku])) {
            foreach ($_SESSION[REINOS_MOLLIE_MAP]['products'][$sku] as $key => $product) {
                if ($product['id'] == $id) {
                    unset($_SESSION[REINOS_MOLLIE_MAP]['products'][$sku][$key]);
                }
            }
        }
    }

    //---------------------------------------------------------------------------------------

    private function _update($sku, $id, $values)
    {
        if (isset($_SESSION[REINOS_MOLLIE_MAP]['products'][$sku])) {
            foreach ($_SESSION[REINOS_MOLLIE_MAP]['products'][$sku] as &$product) {
                if ($product['id'] == $id) {
                    $product = array_merge($product, $values);
                    break;
                }
            }
        }
    }

    //---------------------------------------------------------------------------------------

    private function _find($id)
    {
        $found = [];
        list($sku) = explode('_', $id);

        if (isset($_SESSION[REINOS_MOLLIE_MAP]['products'][$sku])) {
            $found = array_filter($_SESSION[REINOS_MOLLIE_MAP]['products'][$sku], function ($val) use ($id) {
                return $val['id'] == $id;
            });
        }

        if (!empty($found)) {
            return $found[0];
        }
    }

    //---------------------------------------------------------------------------------------

    public function count()
    {
        if (isset($_SESSION[REINOS_MOLLIE_MAP]['products'])) {
            return count($_SESSION[REINOS_MOLLIE_MAP]['products']);
        }

        return 0;
    }

    //---------------------------------------------------------------------------------------

    public function clear()
    {
        $_SESSION[REINOS_MOLLIE_MAP]['products'] = array();
    }

    //---------------------------------------------------------------------------------------

    public function clearAll()
    {
        $_SESSION[REINOS_MOLLIE_MAP] = array();
    }

    //---------------------------------------------------------------------------------------

    public function getItems($prefix = '', $viewFormat = false)
    {
        $shipping_cost = $this->calcTax(
            ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('default_shipping'),
            ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('tax_shipping')
        );

        $disable_shipping = false;

        $return = array(
            $prefix . 'subtotal' => 0,
            $prefix . 'subtotal_plus_tax' => 0,
            $prefix . 'total' => 0,
            $prefix . 'total_plus_tax' => 0,
            $prefix . 'tax' => 0,
            $prefix . 'shipping' => $shipping_cost['price_excl'],
            $prefix . 'shipping_plus_tax' => $shipping_cost['price_incl'],
            $prefix . 'entry_ids' => array(),
            $prefix . 'items' => array()
        );

        if (isset($_SESSION[REINOS_MOLLIE_MAP]['products']) && !empty($_SESSION[REINOS_MOLLIE_MAP]['products'])) {
            //flatten the array as we are coming from a multidim. array
            $items = [];
            foreach ($_SESSION[REINOS_MOLLIE_MAP]['products'] as $products) {
                $items = array_merge($items, $products);
            }

            foreach ($items as $key => $item) {
                $_item = array();

                //prefix
                foreach ($item as $_key => $_value) {
                    //custom fields
                    if ($_key === 'custom_fields' && $_value !== '') {
                        $custom_fields = unserialize(base64_decode($_value));
                        $_value = array();
                        if (!empty($custom_fields)) {
                            foreach ($custom_fields as $field_label => $field_value) {
                                $_value[] = array(
                                    'custom_fields:label' => $field_label,
                                    'custom_fields:value' => $field_value,
                                );
                            }
                        }
                    }

                    $_item[$prefix . $_key] = $_value;
                }

                //check if we found an item with $disable_shipping
                if ($_item[$prefix . 'disable_shipping']) {
                    $disable_shipping = true;
                }

                //save the entry ID
                $return[$prefix . 'entry_ids'][] = $_item[$prefix . 'entry_id'];

                //update totals
                $return[$prefix . 'subtotal'] += $_item[$prefix . 'qty'] * $_item[$prefix . 'price'];
                $return[$prefix . 'subtotal_plus_tax'] += $_item[$prefix . 'qty'] * $_item[$prefix . 'price_plus_tax'];

                //calculate totals
                $_item[$prefix . 'price_total'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($_item[$prefix . 'qty'] * $_item[$prefix . 'price'],
                    $viewFormat);
                $_item[$prefix . 'price_total_plus_tax'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($_item[$prefix . 'qty'] * $_item[$prefix . 'price_plus_tax'],
                    $viewFormat);

                //convert price to correct ones
                $_item[$prefix . 'price'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($_item[$prefix . 'price'],
                    $viewFormat);
                $_item[$prefix . 'price_plus_tax'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($_item[$prefix . 'price_plus_tax'],
                    $viewFormat);

                //save it to the array
                $return[$prefix . 'items'][] = $_item;
            }

            //reset the shipping, as we found an item with disable shipping
            if ($disable_shipping) {
                $return[$prefix . 'shipping'] = 0;
                $return[$prefix . 'shipping_plus_tax'] = 0;
            }

            //add the shipping cost to the subtotal
            $return[$prefix . 'total'] = $return[$prefix . 'subtotal'] + $return[$prefix . 'shipping'];
            $return[$prefix . 'total_plus_tax'] = $return[$prefix . 'subtotal_plus_tax'] + $return[$prefix . 'shipping_plus_tax'];

            //tax
            $return[$prefix . 'tax'] = $return[$prefix . 'total_plus_tax'] - $return[$prefix . 'total'];

            //format price
            $return[$prefix . 'total'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($return[$prefix . 'total'],
                $viewFormat);
            $return[$prefix . 'total_plus_tax'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($return[$prefix . 'total_plus_tax'],
                $viewFormat);
            $return[$prefix . 'subtotal'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($return[$prefix . 'subtotal'],
                $viewFormat);
            $return[$prefix . 'subtotal_plus_tax'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($return[$prefix . 'subtotal_plus_tax'],
                $viewFormat);
            $return[$prefix . 'tax'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($return[$prefix . 'tax'],
                $viewFormat);
            $return[$prefix . 'shipping'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($return[$prefix . 'shipping'],
                $viewFormat);
            $return[$prefix . 'shipping_plus_tax'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($return[$prefix . 'shipping_plus_tax'],
                $viewFormat);

            //convert to a delimiter string
            $return[$prefix . 'entry_ids'] = implode('|', $return[$prefix . 'entry_ids']);
        }

        return $return;
    }

    //---------------------------------------------------------------------------------------

    public function calcTax($price, $tax)
    {
        $price_excl = $price;
        $price_incl = $price;

        if ($this->taxType == 'excl') {
            $taxAmmount = $price * $tax / 100;
            $price_incl = $price + $taxAmmount;
        } else {
            if ($this->taxType == 'incl') {
                $upper = 100 + $tax;
                $taxAmmount = $price / $upper * $tax;
                $price_excl = $price - $taxAmmount;
            }
        }

        return array(
            'price_excl' => $price_excl,
            'price_incl' => $price_incl,

        );
    }

    public function getTax($taxName)
    {
        $tax = ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('tax_' . $taxName);

        // wrong name for the type
        if ($taxName != '' && $tax == '') {
            ee(REINOS_MOLLIE_SERVICE_NAME . ':Log')->add_log(
                'The tax name "' . $taxName . '" cannot be found. The tax name should be one of "high or low". Fallback to a tax of zero',
                'error'
            );

            return 0;
        }

        // did not give a name at all, check the default type
        if ($taxName == '' && $tax == '') {
            return $this->getTax(
                ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('tax_default_type')
            );
        }

        return (int)$tax;
    }
}
