<?php

/**
 * @author        Rein de Vries <support@reinos.nl>
 * @link        http://addons.reinos.nl
 * @copyright    Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * @license     http://addons.reinos.nl/commercial-license
 *
 * Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Rein de Vries and
 * Reinos.nl Internet Media) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */

namespace Reinos\Mollie\Service;

require_once PATH_THIRD . 'reinos_mollie/config.php';

class Currency
{

    public $decPoint = ',';
    public $thousandsSep = '.';

    private $floatFields = [
        'shipping',
        'shipping_plus_tax',
        'subtotal',
        'subtotal_plus_tax',
        'tax',
        'total',
        'total_plus_tax'
    ];

    private $floatFieldsProducts = [
        'price',
        'price_plus_tax',
        'price_total',
        'price_total_plus_tax',
    ];

    public function __construct()
    {
        $this->decPoint = ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('dec_point');
        $this->thousandsSep = ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('thousands_sep');
    }

    public function formatPrice($price = 0, $viewFormat = false)
    {
        if ($viewFormat) {
            return ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('currency_symbol') . '' . number_format($price, 2,
                    $this->decPoint, $this->thousandsSep);
        } else {
            return number_format($price, 2, '.', '');
        }
    }

    public function needFloatConverting($field_name, $value, $viewFormat)
    {
        return $this->convertFloat($this->floatFields, $field_name, $value, $viewFormat);
    }

    public function needFloatConvertingProduct($field_name, $value, $viewFormat)
    {
        return $this->convertFloat($this->floatFieldsProducts, $field_name, $value, $viewFormat);
    }

    private function convertFloat($fields, $field_name, $value, $viewFormat)
    {
        if (in_array($field_name, $fields)) {
            return $this->formatPrice($value, $viewFormat);
        }

        return $value;
    }


}
