<?php

/**
 * @author        Rein de Vries <support@reinos.nl>
 * @link        http://addons.reinos.nl
 * @copyright    Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * @license     http://addons.reinos.nl/commercial-license
 *
 * Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Rein de Vries and
 * Reinos.nl Internet Media) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */

namespace Reinos\Mollie\Service;

require_once PATH_THIRD . 'reinos_mollie/config.php';

class Payment
{

    public $mollie;
    public $payment;

    public function init()
    {
        //Init Mollie
        $this->mollie = new \Mollie\Api\MollieApiClient();
        if (ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('mollie_mode') == 'test') {
            $this->mollie->setApiKey(ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('mollie_test_key'));
        } else {
            $this->mollie->setApiKey(ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('mollie_live_key'));
        }
    }

    public function create($value, $order_id, $hash)
    {
        //create the payment
        $this->payment = $this->mollie->payments->create([
            "amount" => [
                "currency" => "EUR",
                "value" => $value
            ],
            "description" => "Order #" . $order_id,
            "redirectUrl" => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->act_url . '&method=redirect&hash=' . $hash,
            "webhookUrl" => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->act_url . '&method=webhook',
            "metadata" => [
                "order_id" => $order_id,
                "hash" => $hash,
            ],
        ]);

        return $this->payment;
    }

    public function getPaymentForHash($hash)
    {
        $order = ee('Model')->get(REINOS_MOLLIE_MAP . ':Order')->filter('hash', $hash);
        if ($order->count() > 0) {
            return $order->first();
        }

        return false;
    }

    public function getMolliePayment($id)
    {
        return $this->mollie->payments->get($id);
    }

    public function checkStatus($id)
    {
        $payment = $this->mollie->payments->get($id);
        $orderId = $payment->metadata->order_id;

        //get the order
        $order = ee('Model')->get(REINOS_MOLLIE_MAP . ':Order')->filter('order_id', $orderId);
        if ($order->count() > 0) {
            //get the first one
            $order = $order->first();

            //check the statusses
            if ($payment->isPaid() && !$payment->hasRefunds() && !$payment->hasChargebacks()) {
                /*
                 * The payment is paid and isn't refunded or charged back.
                 * At this point you'd probably want to start the process of delivering the product to the customer.
                 */
                $order->status = 'paid';
            } elseif ($payment->isOpen()) {
                /*
                 * The payment is open.
                 */
                $order->status = 'open';
            } elseif ($payment->isPending()) {
                /*
                 * The payment is pending.
                 */
                $order->status = 'pending';
            } elseif ($payment->isFailed()) {
                /*
                 * The payment has failed.
                 */
                $order->status = 'failed';
            } elseif ($payment->isExpired()) {
                /*
                 * The payment is expired.
                 */
                $order->status = 'expired';
            } elseif ($payment->isCanceled()) {
                /*
                 * The payment has been canceled.
                 */
                $order->status = 'canceled';
            } elseif ($payment->hasRefunds()) {
                /*
                 * The payment has been (partially) refunded.
                 * The status of the payment is still "paid"
                 */
                $order->status = 'refunded';
            } elseif ($payment->hasChargebacks()) {
                /*
                 * The payment has been (partially) charged back.
                 * The status of the payment is still "paid"
                 */
                $order->status = 'paid_chargeback';
            } else {
                $order->status = $payment->status;
            }

            $order->save();
            return $order->status;
        }

        return false;
    }
}
