<?php

/**
 * @author        Rein de Vries <support@reinos.nl>
 * @link        http://addons.reinos.nl
 * @copyright    Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * @license     http://addons.reinos.nl/commercial-license
 *
 * Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Rein de Vries and
 * Reinos.nl Internet Media) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */

include(PATH_THIRD . 'reinos_mollie/config.php');


class Reinos_mollie_price_ft extends EE_Fieldtype
{

    public $info = array(
        'name' => REINOS_MOLLIE_TITLE . ' - Price',
        'version' => REINOS_MOLLIE_VERSION,
        'has_global_settings' => 'n'
    );

    public $settings = array();

    public $default_settings = array();

    public $has_array_data = true;

    private $prefix;

    // ----------------------------------------------------------------------

    /**
     * Constructor
     *
     * @access public
     *
     * Calls the parent constructor
     */
    public function __construct()
    {
        parent::__construct();

        $this->prefix = REINOS_MOLLIE_MAP . '_';

        //load lang file
        ee()->lang->loadfile(REINOS_MOLLIE_MAP);

        //require settings
        require 'settings.php';
    }

    // ----------------------------------------------------------------------
    // Before saving the content to the database
    // ----------------------------------------------------------------------

    /**
     * save (Native EE)
     *
     * @access public
     */
    public function save($data)
    {
        return $this->_save($data);
    }

    // ----------------------------------------------------------------------

    /**
     * save
     *
     * @access public
     */
    private function _save($data = '')
    {
        if (strpos($data, '.') === false) {
            $data = $data . '.00';
        }

        return $data;
    }

    // ----------------------------------------------------------------------
    // Display the field for all types
    // ----------------------------------------------------------------------

    /**
     * display_field
     *
     * @access public
     */
    function display_field($data)
    {
        //generate the data
        $type = array_key_exists('fluid_field_data_id', $this->settings) ? 'fluid' : 'default';

        return $this->_display_field($data, $type);
    }

    // ----------------------------------------------------------------------

    /**
     * display_field
     *
     * @access public
     */
    private function _display_field($data, $type = 'default')
    {
        //save data
        $data = trim($data);

        //fieldname
        switch ($type) {
            case 'default':
            default :
                $field_name = $this->field_name;
                break;
        }

        $field_id = $this->settings['field_id'];

        $span = '<span style="position:absolute;padding:7px 0 0 5px;">' . ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('currency_symbol') . '</span>';

        ee()->javascript->output('
				var span = $(\'' . $span . '\').appendTo("body").css({top:-9999});
				var indent = span.width()+4;
				span.remove();
			
			$("#field_id_' . $field_id . '").before(\'' . $span . '\');
			$("#field_id_' . $field_id . '").css({paddingLeft: indent});
			');

        return form_input(array(
            'name' => $field_name,
            'id' => $field_name,
            'class' => REINOS_MOLLIE_MAP . '_price',
            'value' => $data
        ));
    }

    // ----------------------------------------------------------------------
    // Replace the tags for all types
    // ----------------------------------------------------------------------

    /**
     * replace_tag
     *
     * @access public
     */
    public function replace_tag($data, $params = [], $tagdata = false)
    {
        return ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($data);
    }

    /**
     * replace_tag
     *
     * @access public
     */
    public function replace_no_tax($data, $params = [], $tagdata = '')
    {
        $price = ee(REINOS_MOLLIE_SERVICE_NAME . ':Cart')->calcTax($data, 0);
        return ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($price['price_excl'], true);
    }

    /**
     * replace_tag
     *
     * @access public
     */
    public function replace_no_tax_numeric($data, $params = [], $tagdata = '')
    {
        $price = ee(REINOS_MOLLIE_SERVICE_NAME . ':Cart')->calcTax($data, 0);
        return ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($price['price_excl']);
    }

    /**
     * replace_tag
     *
     * @access public
     */
    public function replace_plus_tax($data, $params = [], $tagdata = '')
    {
        $tax = isset($params['tax']) ? $params['tax'] : '';

        if (!is_numeric($tax)) {
            $tax = ee(REINOS_MOLLIE_SERVICE_NAME . ':Cart')->getTax($tax);
        }

        $price = ee(REINOS_MOLLIE_SERVICE_NAME . ':Cart')->calcTax($data, $tax);
        return ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($price['price_incl'], true);
    }

    /**
     * replace_tag
     *
     * @access public
     */
    public function replace_plus_tax_numeric($data, $params = [], $tagdata = '')
    {
        $tax = isset($params['tax']) ? $params['tax'] : '';

        if (!is_numeric($tax)) {
            $tax = ee(REINOS_MOLLIE_SERVICE_NAME . ':Cart')->getTax($tax);
        }

        $price = ee(REINOS_MOLLIE_SERVICE_NAME . ':Cart')->calcTax($data, $tax);
        return ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($price['price_incl']);
    }

    /**
     * replace_tag
     *
     * @access public
     */
    public function replace_numeric($data, $params = [], $tagdata = '')
    {
        return ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($data);
    }

    /**
     * replace_tag
     *
     * @access public
     */
    public function replace_formatted($data, $params = [], $tagdata = '')
    {
        return ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($data, true);
    }

    // ----------------------------------------------------------------------

    /**
     * replace_tag_catchall
     *
     * @access public
     */
    /*function replace_tag_catchall($file_info, $params = array(), $tagdata = FALSE, $modifier)
    {

    }*/

    // ----------------------------------------------------------------------
    // Display the settings for all types
    // ----------------------------------------------------------------------

    /**
     * Display settings screen (Default EE)
     *
     * @access  public
     */
    function display_settings($data)
    {
        $settings_data = $this->_display_settings($data);

        if ($this->content_type() == 'grid') {
            return array('field_options_' . REINOS_MOLLIE_MAP => $settings_data);
        }

        return array(
            'field_options_' . REINOS_MOLLIE_MAP => array(
                'label' => 'field_options',
                'group' => REINOS_MOLLIE_MAP,
                'settings' => $settings_data
            )
        );
    }

    // --------------------------------------------------------------------

    /**
     * Display settings screen
     *
     * @access  public
     */
    // --------------------------------------------------------------------

    /**
     * Display settings screen
     *
     * @access  public
     */
    private function _display_settings($data, $type = '')
    {
        $return = array();

        if (!empty($this->fieldtype_settings)) {
            foreach ($this->fieldtype_settings as $field => $options) {
                //set the override value
                $override_value = false;
                if (isset($options['override'])) {
                    $override_value = ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->get_setting($options['override'] . '_override');
                }

                //set some default options value
                $options['def_value'] = isset($options['def_value']) ? $options['def_value'] : '';
                $options['global'] = isset($options['global']) ? $options['global'] : '';

                //is overrided?
                if ($override_value !== false && $override_value !== '') {
                    $options['value'] = $override_value;

                } //default value
                else {
                    $options['value'] = isset($data[$this->prefix . $options['name']]) ? $data[$this->prefix . $options['name']] : $options['def_value'];
                }

                //global?
                if ($options['global']) {
                    continue;
                }


                //check if we need to load dynamic data
                if (isset($options['choices']) && is_string($options['choices'])) {
                    $split = explode(':', $options['choices']);
                    $serviceName = $split[0];
                    $method = $split[1];

                    if ($method != '') {
                        $options['choices'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':' . $serviceName)->{$method}($options,
                            $this->prefix);
                    }
                }

                if (isset($options['content']) && is_string($options['content'])) {
                    $split = explode(':', $options['choices']);
                    $serviceName = $split[0];
                    $method = $split[1];

                    if ($method != '') {
                        $options['choices'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':' . $serviceName)->{$method}($options,
                            $this->prefix);
                    }
                }

                if ($type == 'native') {
                    $return[] = array(
                        'title' => $options['label'],
                        'desc' => isset($options['desc']) ? $options['desc'] : '',
                        'fields' => $this->generateField($options)
                    );
                } else {
                    if ($type == 'low_var') {
                        $options['field_name'] = $this->prefix . $options['name'];
                        $options['grid'] = false;
                        $options['field'] = $this->generatePerField($options);
                        $return[] = array(
                            $this->prefix . $options['name'],
                            ee('View')->make('ee:_shared/form/field')->render($options)
                        );
                    }
                }
            }
        }

        return $return;
    }

    // ----------------------------------------------------------------------
    // Save the settings for all types
    // ----------------------------------------------------------------------

    /**
     * save_settings (Default EE)
     *
     * @access public
     */
    function save_settings($data)
    {
        $settings = $this->_save_settings($data);

        if (isset($this->field_wide) && $this->field_wide) {
            $settings['field_wide'] = $this->field_wide;
        }

        return $settings;
    }

    // --------------------------------------------------------------------

    /**
     * save_settings
     *
     * @access public
     */
    private function _save_settings($data, $use_data = false)
    {
        $return = array();

        if (!empty($this->fieldtype_settings)) {
            foreach ($this->fieldtype_settings as $field => $options) {
                //global?
                if (isset($options['global']) && $options['global']) {
                    continue;
                }

                $return[$this->prefix . $options['name']] = $use_data ? $data[$this->prefix . $options['name']] : ee()->input->post($this->prefix . $options['name']);
            }
        }

        return $return;
    }

    // ----------------------------------------------------------------------

    /**
     * Validates the field input (EE)
     *
     * @access public
     */
    public function validate($data)
    {
        return $this->_validate($data);
    }

    // ----------------------------------------------------------------------

    /**
     * Validates the field input
     *
     * @access public
     */
    public function _validate($data)
    {
        ee()->load->library('form_validation');

        if (!ee()->form_validation->numeric($data)) {
            return ee()->lang->line('Not a valid value. Please enter the value like "0.00"');
        }

        // Check if number exceeds mysql limits
        if ($data >= 999999.9999) {
            return ee()->lang->line('number_exceeds_limit');
        }

        return true;
    }

    // --------------------------------------------------------------------

    /**
     * Do additional processing after the field is created/modified. (EE)
     *
     * @access public
     */
    public function post_save_settings($data)
    {
        $this->_post_save_settings($data);
    }

    // --------------------------------------------------------------------

    /**
     * Do additional processing after the field is created/modified.
     *
     * @access public
     */
    public function _post_save_settings($data)
    {

    }

    // ----------------------------------------------------------------------

    /**
     * install
     *
     * @access public
     */
    function install()
    {
        $return = array();

        if (!empty($this->fieldtype_settings)) {
            foreach ($this->fieldtype_settings as $field => $options) {
                $return[$this->prefix . $options['name']] = $options['def_value'];
            }
        }

        return $return;
    }

    // ----------------------------------------------------------------------

    /**
     * Update
     */
    function update()
    {
        return true;
    }

    // ----------------------------------------------------------------------

    /**
     * display_global_settings
     *
     * @access public
     */
    function display_global_settings()
    {
        $data = array_merge($this->settings, $_POST);

        $form = '';
        foreach ($this->fieldtype_settings as $field => $options) {
            //global?
            if (!$options['global']) {
                continue;
            }

            switch ($options['type']) {
                //multiselect
                case 'm' :
                    $form .= form_label($options['label'],
                            $options['label']) . NBS . form_multiselect($this->prefix . $options['name'],
                            $options['options'],
                            (isset($data[$this->prefix . $options['name']]) ? $data[$this->prefix . $options['name']] : $options['def_value'])) . NBS . NBS . NBS . ' ';
                    break;

                //select field
                case 's' :
                    $form .= form_label($options['label'],
                            $options['label']) . NBS . form_dropdown($this->prefix . $options['name'],
                            $options['options'],
                            (isset($data[$this->prefix . $options['name']]) ? $data[$this->prefix . $options['name']] : $options['def_value'])) . NBS . NBS . NBS . ' ';
                    break;

                //text field
                default:
                case 't' :
                    $form .= form_label($options['label'],
                            $options['label']) . NBS . form_input($this->prefix . $options['name'],
                            (isset($data[$this->prefix . $options['name']]) ? $data[$this->prefix . $options['name']] : $options['def_value'])) . NBS . NBS . NBS . ' ';
                    break;
            }
        }

        return $form;
    }

    // ----------------------------------------------------------------------

    /**
     * save_global_settings
     *
     * @access public
     */
    function save_global_settings()
    {
        return array_merge($this->settings, $_POST);
    }

    // ----------------------------------------------------------------------

    /**
     * accepts_content_type (GRID EE 2.7)
     *
     * @access public
     */
    public function accepts_content_type($name)
    {
        return ($name == 'channel');
    }

    // --------------------------------------------------------------------

    /**
     * Generate the settings fields
     * @param array $options
     * @return array
     */
    private function generateField($options = array())
    {
        $return = array(
            $this->prefix . $options['name'] => $this->generatePerField($options)
        );

        return $return;
    }

    // ----------------------------------------------------------------------

    /**
     * Generate per field
     * @param array $options
     * @return array
     */
    private function generatePerField($options = array())
    {
        $field_options = $options;

        if (isset($options['options'])) {
            $field_options['choices'] = $options['options'];
        }

        unset($field_options['name']);

        return $field_options;
    }

    // ----------------------------------------------------------------------

    /**
     * Get field setting
     *
     * @param string $name
     * @param bool $default
     * @return mixed|string
     */
    public function get_setting($name = '', $default = false)
    {
        if (isset($this->settings[$name])) {
            return $this->settings[$name];
        }

        return '';
    }

}

