<?php

/**
 * @author        Rein de Vries <support@reinos.nl>
 * @link        http://addons.reinos.nl
 * @copyright    Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * @license     http://addons.reinos.nl/commercial-license
 *
 * Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Rein de Vries and
 * Reinos.nl Internet Media) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */

use EllisLab\ExpressionEngine\Library\CP\Table;

/**
 * Include the config file
 */
require_once PATH_THIRD . 'reinos_mollie/config.php';

class Reinos_mollie_mcp
{

    public $return_data;
    public $settings;

    private $show_per_page = 25;
    private $_base_url;
    private $error_msg;

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->base_url = ee('CP/URL', 'addons/settings/' . REINOS_MOLLIE_MAP);

        //add cp css
        ee(REINOS_MOLLIE_SERVICE_NAME . ':Helper')->mcp_meta_parser('css_custom_path',
            ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->act_url . '&method=cp_css');

        //if there is no valid license, redirect to the license page
        if (REINOS_MOLLIE_LICENSE_ENABLED && !ee(REINOS_MOLLIE_SERVICE_NAME . ':License')->hasValidLicense() && (ee()->uri->segment(5) != 'license' && ee()->uri->segment(5) != 'logs')) {
            ee()->functions->redirect(ee('CP/URL', 'cp/addons/settings/' . REINOS_MOLLIE_MAP . '/license/'));
        }

        //require the default settings
        require PATH_THIRD . REINOS_MOLLIE_MAP . '/settings.php';
    }

    // ----------------------------------------------------------------

    /**
     * Index Function
     *
     * @return    void
     */
    public function index()
    {
        ///show error if needed
        if ($this->error_msg != '') {
            return $this->error_msg;
        }

        //load the view
        if (ee()->session->userdata('group_id') == 1) {
            return $this->settings();
        }

        return $this->orders();
    }


    // ----------------------------------------------------------------

    /**
     * Overview Function
     *
     * @return    void
     */
    public function overview()
    {
        //delete action
        if (isset($_POST['delete']) && !empty($_POST['delete'])) {
            //do your delete stuff here
//			ee('Model')->get('email_form:Submission')->filter('submission_id', ee()->input->post('delete'))->delete();

            //set a message
            ee('CP/Alert')->makeInline(REINOS_MOLLIE_MAP . '_notice')
                ->asSuccess()
                ->withTitle(lang('success'))
                ->addToBody('ID #' . ee()->input->post('delete') . " deleted")
                ->defer();

            ee()->functions->redirect(ee('CP/URL', 'cp/addons/settings/' . REINOS_MOLLIE_MAP . '/' . __FUNCTION__));
            exit;
        }

        //--------------------------
        //custom settings
        $title_page = 'Title Page';
        $action_buttons = array(//REINOS_MOLLIE_MAP.'_delete_logs' => ee('CP/URL', 'addons/settings/'.REINOS_MOLLIE_MAP.'/logs_delete')
        );
        $per_page = 25;
        $sort_col = ee()->input->get('sort_col') ?: 'column_form_id';
        $sort_dir = ee()->input->get('sort_dir') ?: 'desc';
        //end custom settings
        //-------------------------

        // Specify other options
        $table = ee('CP/Table', array(
            'sort_col' => $sort_col,
            'sort_dir' => $sort_dir
        ));

        //set the columns
        $table->setColumns(
            array(
                'column_time',
                'column_username',
                'column_ip',
                'column_service',
                'column_method',
                'column_msg',
                'manage' => array(
                    'type' => Table::COL_TOOLBAR
                )
            )
        );

        //set a no result text
        $table->setNoResultsText('No records available');

        //get all data
        $cur_page = ((int)ee()->input->get('page')) ?: 1;
        $offset = ($cur_page - 1) * $per_page; // Offset is 0 indexed

        $results = ee('Model')->get('webservice:Logs')
            ->order(str_replace('column_', '', $table->config['sort_col']), $table->config['sort_dir'])
            ->limit($per_page)
            ->offset($offset);

        //format the data
        $data = array();
        $ids = array();
        foreach ($results->all() as $result) {
            //save IDS for the delete confirm dialog
            $ids[] = array(
                'id' => $result->id,
                'msg' => 'ID:'
            );

            //set the data
            $data[] = array(
                ee()->localize->human_time($result->time),
                $result->username,
                $result->ip,
                $result->service,
                $result->method,
                $result->msg,
                array(
                    'toolbar_items' => array(
                        'view' => array(
                            'href' => '#',
                            'title' => lang('edit')
                        ),
                        'remove' => array(
                            'href' => '',
                            'title' => lang('remove'),
                            'rel' => "modal-confirm-" . $result->id,
                            'class' => 'm-link'
                        )
                    )
                )
            );
        }

        //set the data
        $table->setData($data);

        // Pass in a base URL to create sorting links
        $base_url = ee('CP/URL', 'addons/settings/' . REINOS_MOLLIE_MAP . '/logs');
        $vars['table'] = $table->viewData($base_url);
        $vars['base_url'] = $vars['table']['base_url'];
        $vars['action_url'] = $base_url;
        $vars['ids'] = $ids;

        //create the paging
        $vars['pagination'] = ee('CP/Pagination', ee('Model')->get('webservice:Logs')->count())
            ->perPage($per_page)
            ->currentPage($cur_page)
            ->render($base_url);

        //Set the title
        $vars['title_page'] = $title_page;

        //set the buttons
        $vars['action_buttons'] = $action_buttons;

        return $this->output('overview', $vars, $title_page);
    }

    // ----------------------------------------------------------------

    /**
     * Settings Function
     *
     * @return    void
     */
    public function settings()
    {
        //is there some data tot save?
        if (isset($_POST) && !empty($_POST)) {
            //validate the form
            $formValidationResult = ee('Validation')
                ->make(array(
                    'frontend_order_url' => 'required',
                    'default_shipping' => 'required|numeric',
                    'tax_high' => 'required|integer|maxLength[2]',
                    'tax_low' => 'integer|maxLength[2]',
                    'tax_shipping' => 'integer|maxLength[2]',
                    'tax_type' => 'required',
                    'tax_default_type' => 'required',
                    'currency_symbol' => 'required',
                    'dec_point' => 'required',
                    'thousands_sep' => 'required',
                    'email_from' => 'required|email',
                    'email_from_name' => 'required',
                    'email_type' => 'required',
                    'mollie_mode' => 'required',
                    'log_severity' => 'required',
                ))
                ->validate($_POST);

            //assign to vars
            $vars['errors'] = $formValidationResult;

            //save only when it is valid
            if ($formValidationResult->isValid()) {
                ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->save_post_settings();
            }
        }

        //set the settings form
        $vars['sections'] = array(
            array(
                array(
                    'title' => REINOS_MOLLIE_MAP . '_frontend_order_url',
                    'desc' => 'Set the frontend Url',
                    'fields' => array(
                        'frontend_order_url' => array(
                            'type' => 'text',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('frontend_order_url'),
                            'required' => true
                        )
                    )
                ),
                array(
                    'title' => REINOS_MOLLIE_MAP . '_default_shipping',
                    'desc' => 'Default Shipping amount',
                    'fields' => array(
                        'default_shipping' => array(
                            'type' => 'text',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('default_shipping'),
                            'required' => true
                        )
                    )
                ),
            ),
            REINOS_MOLLIE_MAP . '_tax_settings' => array(
                array(
                    'title' => REINOS_MOLLIE_MAP . '_tax_high',
                    'desc' => 'TAX percentage (e.g. 21 for 21%)',
                    'fields' => array(
                        'tax_high' => array(
                            'type' => 'text',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('tax_high'),
                            'required' => true
                        )
                    )
                ),
                array(
                    'title' => REINOS_MOLLIE_MAP . '_tax_low',
                    'desc' => 'TAX percentage (e.g. 21 for 21%)',
                    'fields' => array(
                        'tax_low' => array(
                            'type' => 'text',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('tax_low'),
                            'required' => true
                        )
                    )
                ),
                array(
                    'title' => REINOS_MOLLIE_MAP . '_tax_shipping',
                    'desc' => 'TAX percentage (e.g. 21 for 21%)',
                    'fields' => array(
                        'tax_shipping' => array(
                            'type' => 'text',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('tax_shipping'),
                            'required' => true
                        )
                    )
                ),
                array(
                    'title' => REINOS_MOLLIE_MAP . '_tax_default_type',
                    'desc' => 'Set a default TAX.',
                    'fields' => array(
                        'tax_default_type' => array(
                            'type' => 'radio',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('tax_default_type'),
                            'choices' => array(
                                'high' => 'High',
                                'low' => 'Low',
                            ),
                            'required' => true
                        )
                    )
                ),
                array(
                    'title' => REINOS_MOLLIE_MAP . '_tax_type',
                    'desc' => 'Set how the Tax should calculated.',
                    'fields' => array(
                        'tax_type' => array(
                            'type' => 'radio',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('tax_type'),
                            'choices' => array(
                                'none' => 'No Tax',
                                'excl' => 'Exclusive',
                                'incl' => 'Inclusive',
                            ),
                            'required' => true
                        )
                    )
                ),
            ),
            REINOS_MOLLIE_MAP . '_currency_settings' => array(
                array(
                    'title' => REINOS_MOLLIE_MAP . '_currency_symbol',
                    'desc' => 'Set the currency symbol',
                    'fields' => array(
                        'currency_symbol' => array(
                            'type' => 'text',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('currency_symbol'),
                            'required' => true
                        )
                    )
                ),
                array(
                    'title' => REINOS_MOLLIE_MAP . '_dec_point',
                    'desc' => 'Set the decimal point for your currency',
                    'fields' => array(
                        'dec_point' => array(
                            'type' => 'text',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('dec_point'),
                            'required' => true
                        )
                    )
                ),
                array(
                    'title' => REINOS_MOLLIE_MAP . '_thousands_sep',
                    'desc' => 'Set the thousands separator',
                    'fields' => array(
                        'thousands_sep' => array(
                            'type' => 'text',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('thousands_sep'),
                            'required' => true
                        )
                    )
                ),
            ),
            REINOS_MOLLIE_MAP . '_email' => array(
                array(
                    'title' => REINOS_MOLLIE_MAP . '_email_from',
                    'desc' => 'Enter the from email',
                    'fields' => array(
                        'email_from' => array(
                            'type' => 'text',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('email_from'),
                            'required' => true
                        )
                    )
                ),
                array(
                    'title' => REINOS_MOLLIE_MAP . '_email_from_name',
                    'desc' => 'Enter the from name',
                    'fields' => array(
                        'email_from_name' => array(
                            'type' => 'text',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('email_from_name'),
                            'required' => true
                        )
                    )
                ),
                array(
                    'title' => REINOS_MOLLIE_MAP . '_email_bcc',
                    'desc' => 'Enter the BCC email address (comma seperated)',
                    'fields' => array(
                        'email_bcc' => array(
                            'type' => 'text',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('email_bcc'),
                            'required' => false
                        )
                    )
                ),
                array(
                    'title' => REINOS_MOLLIE_MAP . '_email_type',
                    'desc' => 'Select the type of email',
                    'fields' => array(
                        'email_type' => array(
                            'type' => 'radio',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('email_type'),
                            'choices' => array(
                                'text' => 'Text',
                                'html' => 'HTML',
                            ),
                            'required' => true
                        )
                    )
                ),
            ),
            REINOS_MOLLIE_MAP . '_country_settings' => array(
                array(
                    'title' => REINOS_MOLLIE_MAP . '_default_country',
                    'desc' => 'what is the default shipping country',
                    'fields' => array(
                        'default_country' => array(
                            'type' => 'radio',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('default_country'),
                            'choices' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Country')->getCountries(),
                            'required' => false
                        )
                    )
                ),
                array(
                    'title' => REINOS_MOLLIE_MAP . '_allowed_countries',
                    'desc' => 'What are the countries to shipped to',
                    'fields' => array(
                        'allowed_countries' => array(
                            'type' => 'checkbox',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('allowed_countries'),
                            'choices' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Country')->getCountries(),
                            'required' => false
                        )
                    )
                ),
            ),
            REINOS_MOLLIE_MAP . '_mollie_settings' => array(
                array(
                    'title' => REINOS_MOLLIE_MAP . '_mollie_mode',
                    'desc' => 'Are we in the test mode or production mode. None means no payment and will be directly an order. Handy for testing in a local environment.',
                    'fields' => array(
                        'mollie_mode' => array(
                            'type' => 'radio',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('mollie_mode'),
                            'choices' => array(
                                'none' => 'None',
                                'test' => 'Test',
                                'production' => 'Production',
                            ),
                            'required' => true
                        )
                    )
                ),
                array(
                    'title' => REINOS_MOLLIE_MAP . '_mollie_test_key',
                    'fields' => array(
                        'mollie_test_key' => array(
                            'type' => 'text',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('mollie_test_key'),
                            'required' => false
                        )
                    )
                ),
                array(
                    'title' => REINOS_MOLLIE_MAP . '_mollie_live_key',
                    'fields' => array(
                        'mollie_live_key' => array(
                            'type' => 'text',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('mollie_live_key'),
                            'required' => false
                        )
                    )
                ),
            ),
            REINOS_MOLLIE_MAP . '_log_settings' => array(
                array(
                    'title' => REINOS_MOLLIE_MAP . '_log',
                    'desc' => 'What should we log?',
                    'fields' => array(
                        'log_severity' => array(
                            'type' => 'radio',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('log_severity'),
                            'choices' => array(
                                'errors' => 'only errors',
                                'all' => 'Everything',
                                'none' => 'Nothing',
                            ),
                            'required' => true
                        )
                    )
                ),
            )
        );

        // Final view variables we need to render the form
        $vars += array(
            'base_url' => ee('CP/URL', 'cp/addons/settings/' . REINOS_MOLLIE_MAP . '/settings'),
            'cp_page_title' => lang('general_settings'),
            'save_btn_text' => 'btn_save_settings',
            'save_btn_text_working' => 'btn_saving',
            'extra_alerts' => array(
                REINOS_MOLLIE_MAP . '_settings'
            )
        );

        return $this->output('form', $vars, 'settings');
    }

    // ----------------------------------------------------------------

    /**
     * License Function
     *
     * @return array
     */
    public function license()
    {
        //redirect to the login page of reinos
        if (isset($_GET['auth']) && ee()->input->get('auth') == 'yes') {
            $ret = str_replace('&auth=yes', '',
                reduce_double_slashes(ee(REINOS_MOLLIE_SERVICE_NAME . ':Helper')->currentUrl()));
            $data = json_encode(array(
                'return' => $ret,
                'license_key' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('license_key'),
                'module_name' => str_replace('reinos_', '', REINOS_MOLLIE_MAP)
            ));
            ee()->functions->redirect('https://addons.reinos.nl/auth?data=' . base64_encode($data));
        }

        //login successfull from addons.reinos.nl
        if (isset($_GET['auth']) && (ee()->input->get('auth') == 'success' || ee()->input->get('auth') == 'failed')) {
            //save the member ID
            ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->save_setting('license_reinos_member_id',
                ee()->input->get('member_id'));
            ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->set_setting('license_reinos_member_id',
                ee()->input->get('member_id'));

            //check the license with the server
            $license_check = ee(REINOS_MOLLIE_SERVICE_NAME . ':License')->checkLicense(ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('license_key'),
                ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('license_reinos_member_id'));

            //License check okay?
            if ($license_check->success) {
                //set a message
                ee('CP/Alert')->makeInline(REINOS_MOLLIE_MAP . '_settings')
                    ->addToBody(ee()->lang->line('preferences_updated'))
                    ->asSuccess()
                    ->withTitle(lang('success'))
                    ->addToBody($license_check->message)
                    ->now();
            }
        }

        //is there some data tot save?
        if (isset($_POST) && !empty($_POST)) {
            //validate the form
            $formValidationResult = ee('Validation')
                ->make(array(
                    'license_key' => 'required'
                ))
                ->validate($_POST);

            //assign to vars
            $vars['errors'] = $formValidationResult;

            //save only when it is valid
            if ($formValidationResult->isValid()) {
                //save the settings
                ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->save_post_settings('/license&auth=yes');
            }
        }

        //License not correct?
        if (!ee(REINOS_MOLLIE_SERVICE_NAME . ':License')->hasValidLicense()) {
            //set a message
            ee('CP/Alert')->makeInline(REINOS_MOLLIE_MAP . '_license')
                ->asIssue()
                ->withTitle(lang('error'))
                ->addToBody('You have an incorrect license. Enter a valid license in order to activate the addon.')
                ->now();
        }

        //set the settings form
        $vars['sections'] = array(
            array(
                array(
                    'title' => REINOS_MOLLIE_MAP . '_license_key',
                    'desc' => 'Your license key',
                    'fields' => array(
                        'license_key' => array(
                            'type' => 'text',
                            'value' => ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('license_key'),
                            'required' => true
                        )
                    )
                ),
                array(
                    'title' => '',
                    'fields' => array(
                        'info' => array(
                            'type' => 'html',
                            'content' => '
                                <div class="reinos-license_status">
                                    <div class="reinos-license-info" id="' . REINOS_MOLLIE_MAP . '_license_status">
                                        <span class="st-closed invalid_license" style="display: none;">Invalid license</span>
                                        <span class="st-closed unlicensed" style="display: none;">Unlicensed</span>
                                        <span class="st-info valid_license" style="display: none;">Valid license</span>
                                    </div>
                                   
                                    <h4>License info</h4>
                                    <ul>
                                        <li>Every valid license will work on every local development <i>(*.local, *.test, *.dev or *.localhost)</i>. </li>
                                        <li>
                                            In order to let your license work on your domain, <strong>you have to set the production url</strong> and/or test url. This can be done in you account on
                                            <a href="http://addons.reinos.nl/profile/licenses" target="_blank">addons.reinos.nl</a>.
                                        </li>
                                        <li>
                                            If you run a MSM site, you have to enter all your MSM sites in your account on <a href="http://addons.reinos.nl/profile/licenses" target="_blank">addons.reinos.nl</a>.
                                        </li>
                                        <li>
                                            Running with an invalid license, will <strong>cease the addon to work</strong>.
                                        </li>
                                        <li>If you bought a license in the EE store, it will transferred also to your account on <a href="http://addons.reinos.nl/profile/licenses" target="_blank">addons.reinos.nl</a></li>
                                        <li>By using this software, you agree to the <a href="https://addons.reinos.nl/commercial-license" target="_blank">Add-on License Agreement.</a></li>
                                    </ul>
                                </div>
                                '
                        )
                    )
                )
            )
        );

        // Final view variables we need to render the form
        $vars += array(
            'base_url' => ee('CP/URL', 'cp/addons/settings/' . REINOS_MOLLIE_MAP . '/license'),
            'cp_page_title' => lang(REINOS_MOLLIE_MAP . '_license_settings'),
            'save_btn_text' => 'save and auth license',
            'save_btn_text_working' => 'btn_saving',
            'extra_alerts' => array(
                REINOS_MOLLIE_MAP . '_settings',
                REINOS_MOLLIE_MAP . '_license',
            )
        );

        return $this->output('form', $vars, 'settings');
    }

    // ----------------------------------------------------------------

    /**
     * Overview Function
     *
     * @return    void
     */
    public function logs()
    {
        //delete action
        if (isset($_POST['delete']) && !empty($_POST['delete'])) {
            //do your delete stuff here
            ee('Model')->get(REINOS_MOLLIE_SERVICE_NAME . ':Log')->filter('log_id',
                ee()->input->post('delete'))->delete();

            //set a message
            ee('CP/Alert')->makeInline(REINOS_MOLLIE_MAP . '_notice')
                ->asSuccess()
                ->withTitle(lang('success'))
                ->addToBody('Log ID #' . ee()->input->post('delete') . " deleted")
                ->defer();

            ee()->functions->redirect(ee('CP/URL', 'cp/addons/settings/' . REINOS_MOLLIE_MAP . '/' . __FUNCTION__));
            exit;
        }

        //--------------------------
        //custom settings
        $title_page = 'Log overview';
        $action_buttons = array(
            REINOS_MOLLIE_MAP . '_delete_all_logs' => ee('CP/URL',
                'cp/addons/settings/' . REINOS_MOLLIE_MAP . '/delete_all_logs')
        );
        $per_page = 25;
        $sort_col = ee()->input->get('sort_col') ?: 'column_log_id';
        $sort_dir = ee()->input->get('sort_dir') ?: 'desc';
        //end custom settings
        //-------------------------

        // Specify other options
        $table = ee('CP/Table', array(
            'sort_col' => $sort_col,
            'sort_dir' => $sort_dir
        ));

        //set the columns
        $table->setColumns(
            array(
                'column_log_id',
                'column_severity',
                'column_time',
                'column_message',
                'manage' => array(
                    'type' => Table::COL_TOOLBAR
                )
            )
        );

        //set a no result text
        $table->setNoResultsText('No logs available');

        //get all data
        $cur_page = ((int)ee()->input->get('page')) ?: 1;
        $offset = ($cur_page - 1) * $per_page; // Offset is 0 indexed

        $results = ee('Model')->get(REINOS_MOLLIE_SERVICE_NAME . ':Log')
            ->order(str_replace('column_', '', $table->config['sort_col']), $table->config['sort_dir'])
            ->limit($per_page)
            ->filter('site_id', ee()->config->item('site_id'))
            ->offset($offset);

        //format the data
        $data = array();
        $ids = array();
        foreach ($results->all() as $result) {
            //save IDS for the delete confirm dialog
            $ids[] = array(
                'id' => $result->log_id,
                'msg' => 'ID:'
            );

            //set the data
            $data[] = array(
                $result->log_id,
                $result->severity,
                ee()->localize->human_time($result->time),
                $result->message,
                array(
                    'toolbar_items' => array(
                        'remove' => array(
                            'href' => '',
                            'title' => lang('remove'),
                            'rel' => "modal-confirm-" . $result->log_id,
                            'class' => 'm-link'
                        )
                    )
                )
            );
        }

        //set the data
        $table->setData($data);

        // Pass in a base URL to create sorting links
        $base_url = ee('CP/URL', 'addons/settings/' . REINOS_MOLLIE_MAP . '/logs');
        $vars['table'] = $table->viewData($base_url);
        $vars['base_url'] = $vars['table']['base_url'];
        $vars['action_url'] = $base_url;
        $vars['ids'] = $ids;

        //create the paging
        $vars['pagination'] = ee('CP/Pagination', $results->count())
            ->perPage($per_page)
            ->currentPage($cur_page)
            ->render($base_url);

        //Set the title
        $vars['title_page'] = $title_page;

        //set the buttons
        $vars['action_buttons'] = $action_buttons;

        return $this->output('overview', $vars, $title_page);
    }

    /**
     * @return array
     */
    public function delete_all_logs()
    {
        //delete member
        if (ee()->input->post('confirm') == 'ok') {
            $result = ee('Model')->get(REINOS_MOLLIE_SERVICE_NAME . ':Log')->all();
            $result->delete();

            //set a message
            ee('CP/Alert')->makeInline(REINOS_MOLLIE_MAP . '_notice')
                ->asSuccess()
                ->withTitle(lang('success'))
                ->addToBody('Logs deleted')
                ->defer();

            ee()->functions->redirect(ee('CP/URL', 'cp/addons/settings/' . REINOS_MOLLIE_MAP . '/logs/'));
        }

        $vars = array();
        $vars['title_page'] = 'Delete all Logs';
        $vars['form_url'] = ee('CP/URL', 'cp/addons/settings/' . REINOS_MOLLIE_MAP . '/delete_all_logs/');

        return $this->output('delete', $vars, $vars['title_page']);
    }

    /**
     * Overview Function
     *
     */
    public function orders($order_id = 0)
    {
        if ($order_id > 0) {
            return $this->view_order($order_id);
        }

        //--------------------------
        //custom settings
        $title_page = 'Order overview';
        $action_buttons = array(
            REINOS_MOLLIE_MAP . '_export_all_orders' => ee('CP/URL',
                'cp/addons/settings/' . REINOS_MOLLIE_MAP . '/export_all_orders')
        );
        $per_page = 25;
        $sort_col = ee()->input->get('sort_col') ?: 'column_order_id';
        $sort_dir = ee()->input->get('sort_dir') ?: 'desc';
        //end custom settings
        //-------------------------

        // Specify other options
        $table = ee('CP/Table', array(
            'sort_col' => $sort_col,
            'sort_dir' => $sort_dir
        ));

        //set the columns
        $table->setColumns(
            array(
                'column_order_id',
                'column_customer_first_name',
                'column_customer_last_name',
                'column_total_plus_tax',
                'column_time',
                'column_status',
                'manage' => array(
                    'type' => Table::COL_TOOLBAR
                )
            )
        );

        //set a no result text
        $table->setNoResultsText('No Orders');

        //get all data
        $cur_page = ((int)ee()->input->get('page')) ?: 1;
        $offset = ($cur_page - 1) * $per_page; // Offset is 0 indexed

        $results = ee('Model')->get(REINOS_MOLLIE_SERVICE_NAME . ':Order')
            ->order(str_replace('column_', '', $table->config['sort_col']), $table->config['sort_dir'])
            ->limit($per_page)
            ->filter('site_id', ee()->config->item('site_id'))
            ->offset($offset);

        //format the data
        $data = array();
        $ids = array();
        foreach ($results->all() as $result) {
            //save IDS for the delete confirm dialog
            $ids[] = array(
                'id' => $result->order_id,
                'msg' => 'ID:'
            );

            $frontend_order_url = ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('frontend_order_url');
            if ($frontend_order_url != '') {
                $order_detail_url = reduce_double_slashes(ee()->config->item('site_url') . '/' . $frontend_order_url . '/' . $result->hash);
                $target = '_blank';
            } else {
                $order_detail_url = ee('CP/URL',
                    'cp/addons/settings/' . REINOS_MOLLIE_MAP . '/orders/' . $result->order_id);
                $target = '';
            }

            //set the data
            $data[] = array(
                $result->order_id,
                $result->customer_first_name,
                $result->customer_last_name,
                ee(REINOS_MOLLIE_SERVICE_NAME . ':Currency')->formatPrice($result->total_plus_tax, true),
                ee()->localize->human_time($result->time),
                $result->status,
                array(
                    'toolbar_items' => array(
                        'view' => array(
                            'href' => $order_detail_url,
                            'title' => lang('view'),
                            'target' => $target
                        )
                    )
                )
            );
        }

        //set the data
        $table->setData($data);

        // Pass in a base URL to create sorting links
        $base_url = ee('CP/URL', 'addons/settings/' . REINOS_MOLLIE_MAP . '/orders');
        $vars['table'] = $table->viewData($base_url);
        $vars['base_url'] = $vars['table']['base_url'];
        $vars['action_url'] = $base_url;
        $vars['ids'] = $ids;

        //create the paging
        $total_orders = ee('Model')->get(REINOS_MOLLIE_SERVICE_NAME . ':Order')->count();
        $vars['pagination'] = ee('CP/Pagination', $total_orders)
            ->perPage($per_page)
            ->currentPage($cur_page)
            ->render($base_url);

        //Set the title
        $vars['title_page'] = $title_page;

        //set the buttons
        $vars['action_buttons'] = $action_buttons;

        return $this->output('overview', $vars, $title_page);
    }

    public function export_all_orders()
    {
        $orders = ee('Model')->get(REINOS_MOLLIE_MAP . ':Order');
        $csv = ee('CSV');

        if ($orders->count() > 0) {
            foreach ($orders->all() as $item) {
                $item_values = $item->toArray();
                $item_values['date'] = ee()->localize->human_time($item_values['time']);
                $item_values['return_url'] = reduce_double_slashes($item_values['return_url'] . '/' . $item_values['hash']);

                $item_values['orderder_items'] = '';

                $product_items = $item_values['items'];

                //remove the items
                unset($item_values['items']);

                foreach ($product_items as $orderd_item) {
                    $item_values['ordered_product_name'] = $orderd_item['name'];
                    $item_values['ordered_product_qty'] = $orderd_item['qty'];
                    $item_values['ordered_product_price'] = $orderd_item['price'];
                    $item_values['ordered_product_price_plus_tax'] = $orderd_item['price_plus_tax'];
                    $item_values['ordered_product_price_total'] = $orderd_item['qty'] * $orderd_item['price'];
                    $item_values['ordered_product_price_total_plus_tax'] = $orderd_item['qty'] * $orderd_item['price_plus_tax'];
                }

                $csv->addRow($item_values);
            }

            $cache_path = ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('cache_path') . '/' . time() . '_export_orders.csv';
            $csv->save($cache_path);

            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Disposition: attachment; filename=' . basename($cache_path));
            header('Expires: 0');
            header('Cache-Control: must-revalidate');
            header('Pragma: public');
            header('Content-Length: ' . filesize($cache_path));
            readfile($cache_path);
            exit;
        } else {
            ee()->functions->redirect(ee('CP/URL', 'cp/addons/settings/' . REINOS_MOLLIE_MAP . '/orders'));
        }
    }

    // ----------------------------------------------------------------

    /**
     * Overview Function
     *
     * @return    void
     */
    public function view_order($id = 0)
    {
        //no id?
        if ($id == 0) {
            ee()->functions->redirect(ee('CP/URL', 'cp/addons/settings/' . REINOS_MOLLIE_MAP . '/orders'));
        }

        //--------------------------
        //custom settings
        $title_page = 'View order #' . $id;
        //end custom settings
        //-------------------------

        //get the submission data
        $vars['result'] = ee('Model')->get(REINOS_MOLLIE_SERVICE_NAME . ':Order')->filter('order_id', $id)->first();

        $vars['title_page'] = $title_page;
        return $this->output('order', $vars, $title_page);
    }

    // ----------------------------------------------------------------

    /**
     * Overview Function
     *
     * @return    void
     */
    public function example_templates($install = '')
    {
        if ($install == 'install') {
            $dest = SYSPATH . 'user/templates/' . ee()->config->item('site_short_name') . '/mollie_store.group/';
            $src = PATH_THIRD . 'reinos_mollie/templates/store.group';

            //copy the files
            ee('Filesystem')->copy($src, $dest);

            //set a message
            ee('CP/Alert')->makeInline(REINOS_MOLLIE_MAP . '_notice')
                ->asSuccess()
                ->withTitle(lang('success'))
                ->addToBody('Templates installed')
                ->defer();

            ee()->functions->redirect(ee('CP/URL', 'cp/addons/settings/' . REINOS_MOLLIE_MAP . '/' . __FUNCTION__));
            exit;
        }
        //no id?
//		if($id == 0)
//		{
//			ee()->functions->redirect(ee('CP/URL', 'cp/addons/settings/'.REINOS_MOLLIE_MAP.'/orders'));
//		}

        //--------------------------
        //custom settings
        $title_page = 'Example Templates';
        //end custom settings
        //-------------------------

        $vars['title_page'] = $title_page;
        return $this->output('example_templates', $vars, $title_page);
    }

    /**
     * Overview Function
     *
     * @return    void
     */
    public function migrate($start = '')
    {
        if ($start == 'start') {
            // migrate the settings
            $mollieSettingsQuery = ee('Model')->get(REINOS_MOLLIE_SERVICE_NAME.':Settings')->all();
            $simpleStoreFields = [
                'log_severity',
                'email_from',
                'email_from_name',
                'email_bcc',
                'email_type',
                'default_country',
                'allowed_countries',
                'default_shipping',
                'tax_high',
                'tax_low',
                'tax_shipping',
                'tax_type',
                'tax_default_type',
                'frontend_order_url',
                'dec_point',
                'thousands_sep',
            ];

            foreach($mollieSettingsQuery as $setting) {
                if(in_array($setting->var, $simpleStoreFields)) {
                    ee()->db->where('var', $setting->var)->update('reinos_simple_commerce_settings', [
                        'value' => $setting->value,
                    ]);
                }
            }

            // migrate the orders
            $ordersQuery = ee('Model')->get(REINOS_MOLLIE_SERVICE_NAME.':Order')->all();
            foreach($ordersQuery as $order) {
                ee()->db->insert('reinos_simple_commerce_orders', [
                    'order_id' => $order->order_id,
                    'hash' => $order->hash,
                    'site_id' => $order->site_id,
                    'billing_address' => $order->billing_address,
                    'billing_address2' => $order->billing_address2,
                    'billing_city' => $order->billing_city,
                    'billing_country' => $order->billing_country,
                    'billing_zip' => $order->billing_zip,
                    'shipping_address' => $order->shipping_address,
                    'shipping_address2' => $order->shipping_address2,
                    'shipping_city' => $order->shipping_city,
                    'shipping_country' => $order->shipping_country,
                    'shipping_zip' => $order->shipping_zip,
                    'customer_first_name' => $order->customer_first_name,
                    'customer_last_name' => $order->customer_last_name,
                    'customer_company' => $order->customer_company,
                    'customer_email' => $order->customer_email,
                    'customer_phone' => $order->customer_phone,
                    'shipping' => $order->shipping,
                    'shipping_plus_tax' => $order->shipping_plus_tax,
                    'subtotal' => $order->subtotal,
                    'subtotal_plus_tax' => $order->subtotal_plus_tax,
                    'tax' => $order->tax,
                    'total' => $order->total,
                    'total_plus_tax' => $order->total_plus_tax,
                    'transaction_id' => $order->transaction_id,
                    'payment_url' => $order->payment_url,
                    'return_url' => $order->return_url,
                    'checkout_url' => $order->checkout_url,
                    'status' => $order->status,
                    'items' => $order->items,
                    'time' => $order->time,
                    'fields' => $order->fields,
                ]);
            }

            // convert the field to the Simple Store field
            ee()->db->where('field_type', 'reinos_mollie_price')->update('channel_fields', [
                'field_type' => 'reinos_simple_commerce_price'
            ]);

            //set a message
            ee('CP/Alert')->makeInline(REINOS_MOLLIE_MAP . '_notice')
                ->asSuccess()
                ->withTitle(lang('success'))
                ->addToBody('Mirgation done')
                ->defer();

            ee()->functions->redirect(ee('CP/URL', 'cp/addons/settings/' . REINOS_MOLLIE_MAP . '/' . __FUNCTION__));
            exit;
        }
        //--------------------------
        //custom settings
        $title_page = 'Migrate to Simple Store';
        //end custom settings
        //-------------------------

        $vars['title_page'] = $title_page;
        return $this->output('migrate', $vars, $title_page);
    }

    // ----------------------------------------------------------------

    /**
     * Run a specific update file
     */
    public function update()
    {
        if (in_array(ee()->input->get('version'), $this->updates)) {
            ee(REINOS_MOLLIE_SERVICE_NAME . ':Installer')->init_update(ee()->input->get('version'));
            die('updater ' . ee()->input->get('version') . ' executed');
        }

        die('Cannot run updater ' . ee()->input->get('version'));
    }

    // ----------------------------------------------------------------

    /**
     * call any act route from the cp
     */
    public function ajax()
    {
        include 'mod.' . REINOS_MOLLIE_MAP . '.php';
        $mod = new Reinos_mollie();
        $mod->act_route();
    }

    // ----------------------------------------------------------------

    /**
     * Delete Dorm
     *
     * @param int $form_id
     * @return array
     */
//	public function delete_form($id = 0)
//	{
//		//delete member
//		if(ee()->input->post('confirm') == 'ok')
//		{
//			$form = ee('Model')->get('email_form:Form')->filter('form_id', $form_id)->first();
//			$form->delete();
//
//			//set a message
//			ee('CP/Alert')->makeInline(EMAIL_QUEUE_MAP.'_notice')
//				->asSuccess()
//				->withTitle(lang('success'))
//				->addToBody('Form deleted')
//				->defer();
//
//			ee()->functions->redirect(ee('CP/URL', 'cp/addons/settings/'.EMAIL_QUEUE_MAP.'/forms/'));
//		}
//
//		$vars = array();
//		$vars['title_page'] = 'Delete form';
//      $vars['form_url'] = ee('CP/URL', 'cp/addons/settings/'.EMAIL_QUEUE_MAP.'/delete_form/'.$id);
//
//		return $this->output('delete', $vars, $vars['title_page']);
//	}

    // ----------------------------------------------------------------

    private function output($template, $vars, $heading = '')
    {
        //support for sidebar?
        $sidebar = ee('CP/Sidebar')->make();

        if (ee()->session->userdata('group_id') == 1) {
            $sidebar->addHeader(lang(REINOS_MOLLIE_MAP . '_settings'),
                ee('CP/URL', 'addons/settings/' . REINOS_MOLLIE_MAP . '/settings'));
            $sidebar->addHeader(lang(REINOS_MOLLIE_MAP . '_example_templates'),
                ee('CP/URL', 'addons/settings/' . REINOS_MOLLIE_MAP . '/example_templates'));
            $sidebar->addHeader(lang(REINOS_MOLLIE_MAP . '_log'),
                ee('CP/URL', 'addons/settings/' . REINOS_MOLLIE_MAP . '/logs'));
            $sidebar->addHeader(lang(REINOS_MOLLIE_MAP . '_migrate'),
                                ee('CP/URL', 'addons/settings/' . REINOS_MOLLIE_MAP . '/migrate'));
        }

        $sidebar->addHeader(lang(REINOS_MOLLIE_MAP . '_orders'),
            ee('CP/URL', 'addons/settings/' . REINOS_MOLLIE_MAP . '/orders'));

        if (REINOS_MOLLIE_LICENSE_ENABLED) {
            $sidebar->addHeader(lang(REINOS_MOLLIE_MAP . '_license'),
                ee('CP/URL', 'addons/settings/' . REINOS_MOLLIE_MAP . '/license'));
        }

        return array(
            'body' => ee('View')->make(REINOS_MOLLIE_SERVICE_NAME . ':' . $template)->render($vars),
            'heading' => REINOS_MOLLIE_TITLE . ' - ' . lang($heading)
        );
    }

    // ----------------------------------------------------------------


}
